@@ -1,6 +1,29 @@
-
 #ifndef LIBCURL_CONFIG_AMIGAOS_H
 #define LIBCURL_CONFIG_AMIGAOS_H
+/***************************************************************************
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
+ *                             \___|\___/|_| \_\_____|
+ *
+ * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
+ *
+ * This software is licensed as described in the file COPYING, which
+ * you should have received as part of this distribution. The terms
+ * are also available at http://curl.haxx.se/docs/copyright.html.
+ *
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the COPYING file.
+ *
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
+ *
+ * $Id: config-amigaos.h,v 1.12 2007-02-28 14:45:49 yangtse Exp $
+ ***************************************************************************/
+
+#ifdef __AMIGA__ /* Any AmigaOS flavour */
 
 #define HAVE_ARPA_INET_H 1
 #define HAVE_GETHOSTBYADDR 1
@@ -31,6 +54,8 @@
 #define HAVE_SETJMP_H 1
 #define HAVE_SGTTY_H 1
 #define HAVE_SIGNAL 1
+#define HAVE_SIGNAL_H 1
+#define HAVE_SIG_ATOMIC_T 1
 #define HAVE_SOCKET 1
 #define HAVE_STRCASECMP 1
 #define HAVE_STRDUP 1
@@ -39,6 +64,7 @@
 #define HAVE_STRINGS_H 1
 #define HAVE_STRING_H 1
 #define HAVE_STRSTR 1
+#define HAVE_STRUCT_TIMEVAL 1
 #define HAVE_SYS_PARAM_H 1
 #define HAVE_SYS_SOCKET_H 1
 #define HAVE_SYS_SOCKIO_H 1
@@ -113,4 +139,5 @@
 #define SEND_TYPE_ARG4 int
 #define SEND_TYPE_RETV int
 
+#endif /* __AMIGA__ */
 #endif /* LIBCURL_CONFIG_AMIGAOS_H */
