@@ -15,6 +15,7 @@
 #define HAVE_TIME_H             1
 #define HAVE_STDLIB_H           1
 #define HAVE_UTIME_H            1
+#define HAVE_SYS_TIME_H         1
 
 #define TIME_WITH_SYS_TIME      1
 
@@ -27,9 +28,12 @@
 #define HAVE_MEMCPY             1
 #define HAVE_SELECT             1
 #define HAVE_SOCKET             1
+#define HAVE_STRUCT_TIMEVAL     1
 
 //#define HAVE_STRICMP          1
 #define HAVE_SIGACTION          1
+#define HAVE_SIGNAL_H           1
+#define HAVE_SIG_ATOMIC_T       1
 
 #ifdef MACOS_SSL_SUPPORT
 #       define USE_SSLEAY       1
@@ -43,6 +47,8 @@
 
 #define HAVE_FIONBIO            1
 
+#define RETSIGTYPE void
+
 #define HAVE_GETNAMEINFO 1
 #define GETNAMEINFO_QUAL_ARG1 const
 #define GETNAMEINFO_TYPE_ARG1 struct sockaddr *
