@@ -9,9 +9,6 @@
 /*                          HEADER FILES                            */
 /* ---------------------------------------------------------------- */
 
-/* Define if you have the <alloca.h> header file.  */
-/* #define HAVE_ALLOCA_H 1 */
-
 /* Define if you have the <arpa/inet.h> header file.  */
 /* #define HAVE_ARPA_INET_H 1 */
 
@@ -36,9 +33,6 @@
 /* Define if you have the <io.h> header file.  */
 #define HAVE_IO_H 1
 
-/* Define if you have the <malloc.h> header file.  */
-#define HAVE_MALLOC_H 1
-
 /* Define if you need the malloc.h header header file even with stdlib.h  */
 #define NEED_MALLOC_H 1
 
@@ -48,6 +42,9 @@
 /* Define if you have the <netinet/in.h> header file.  */
 /* #define HAVE_NETINET_IN_H 1 */
 
+/* Define if you have the <signal.h> header file. */
+#define HAVE_SIGNAL_H 1
+
 /* Define if you have the <sgtty.h> header file.  */
 /* #define HAVE_SGTTY_H 1 */
 
@@ -75,6 +72,9 @@
 /* Define if you have the <sys/stat.h> header file.  */
 #define HAVE_SYS_STAT_H 1
 
+/* Define if you have the <sys/time.h> header file */
+/* #define HAVE_SYS_TIME_H 1 */
+
 /* Define if you have the <sys/types.h> header file.  */
 /* #define HAVE_SYS_TYPES_H 1 */
 
@@ -111,6 +111,9 @@
 /*                        OTHER HEADER INFO                         */
 /* ---------------------------------------------------------------- */
 
+/* Define if sig_atomic_t is an available typedef. */
+#define HAVE_SIG_ATOMIC_T 1
+
 /* Define if you have the ANSI C header files.  */
 #define STDC_HEADERS 1
 
@@ -195,15 +198,9 @@
 /* Define if you have the tcsetattr function.  */
 /* #define HAVE_TCSETATTR 1 */
 
-/* Define if you have the uname function.  */
-/* #define HAVE_UNAME 1 */
-
 /* Define if you have the utime function */
 #define HAVE_UTIME 1
 
-/* Define if you have the vprintf function.  */
-#define HAVE_VPRINTF 1
-
 /* Define if you have the getnameinfo function. */
 #define HAVE_GETNAMEINFO 1
 
@@ -269,13 +266,15 @@
 #define in_addr_t unsigned long
 
 /* Define as the return type of signal handlers (int or void).  */
-/* #define RETSIGTYPE void */
+#define RETSIGTYPE void
 
-/* Define to `unsigned' if size_t is not an available 'typedefed' type */
-/* #define size_t unsigned */
-
-/* Define to 'int' if ssize_t is not an available 'typedefed' type */
+/* Define ssize_t if it is not an available 'typedefed' type */
+#if (defined(__WATCOMC__) && (__WATCOMC__ >= 1240)) || defined(__POCC__)
+#elif defined(_WIN64)
+#define ssize_t __int64
+#else
 #define ssize_t int
+#endif
 
 /* Define to 'int' if socklen_t is not an available 'typedefed' type */
 #ifndef HAVE_WS2TCPIP_H
@@ -307,6 +306,9 @@
 /* Define this if you have struct sockaddr_storage */
 /* #define HAVE_STRUCT_SOCKADDR_STORAGE 1 */
 
+/* Define this if you have struct timeval */
+#define HAVE_STRUCT_TIMEVAL 1
+
 /* ---------------------------------------------------------------- */
 /*                        COMPILER SPECIFIC                         */
 /* ---------------------------------------------------------------- */
