@@ -20,12 +20,17 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: hostip.h,v 1.54 2006-07-24 15:48:48 giva Exp $
+ * $Id: hostip.h,v 1.57 2006-09-08 05:18:07 yangtse Exp $
  ***************************************************************************/
 
 #include "setup.h"
 #include "hash.h"
 
+#if (defined(NETWARE) && defined(__NOVELL_LIBC__))
+#undef in_addr_t
+#define in_addr_t uint32_t
+#endif
+
 /*
  * Setup comfortable CURLRES_* defines to use in the host*.c sources.
  */
@@ -82,7 +87,8 @@
 #define CURL_ASYNC_SUCCESS ARES_SUCCESS
 #else
 #define CURL_ASYNC_SUCCESS CURLE_OK
-#define ares_cancel(x)
+#define ares_cancel(x) do {} while(0)
+#define ares_destroy(x) do {} while(0)
 #endif
 
 /*
