@@ -1,16 +1,16 @@
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: llist.c,v 1.14 2004/05/11 11:30:23 bagder Exp $
+ * $Id: llist.c,v 1.18 2006-10-25 07:19:45 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -32,8 +32,8 @@
 /* this must be the last include file */
 #include "memdebug.h"
 
-void 
-Curl_llist_init(curl_llist *l, curl_llist_dtor dtor)
+void
+Curl_llist_init(struct curl_llist *l, curl_llist_dtor dtor)
 {
   l->size = 0;
   l->dtor = dtor;
@@ -41,12 +41,12 @@ Curl_llist_init(curl_llist *l, curl_llist_dtor dtor)
   l->tail = NULL;
 }
 
-curl_llist *
+struct curl_llist *
 Curl_llist_alloc(curl_llist_dtor dtor)
 {
-  curl_llist *list;
+  struct curl_llist *list;
 
-  list = (curl_llist *)malloc(sizeof(curl_llist));
+  list = (struct curl_llist *)malloc(sizeof(struct curl_llist));
   if(NULL == list)
     return NULL;
 
@@ -59,10 +59,11 @@ Curl_llist_alloc(curl_llist_dtor dtor)
  * Curl_llist_insert_next() returns 1 on success and 0 on failure.
  */
 int
-Curl_llist_insert_next(curl_llist *list, curl_llist_element *e, const void *p)
+Curl_llist_insert_next(struct curl_llist *list, struct curl_llist_element *e,
+                       const void *p)
 {
-  curl_llist_element *ne =
-    (curl_llist_element *) malloc(sizeof(curl_llist_element));
+  struct curl_llist_element *ne =
+    (struct curl_llist_element *) malloc(sizeof(struct curl_llist_element));
   if(!ne)
     return 0;
 
@@ -90,37 +91,9 @@ Curl_llist_insert_next(curl_llist *list, curl_llist_element *e, const void *p)
   return 1;
 }
 
-#if 0
-int 
-Curl_llist_insert_prev(curl_llist *list, curl_llist_element *e, const void *p)
-{
-  curl_llist_element *ne;
-
-  ne = (curl_llist_element *) malloc(sizeof(curl_llist_element));
-  ne->ptr = (void *) p;
-  if (list->size == 0) {
-    list->head = ne;
-    list->head->prev = NULL;
-    list->head->next = NULL;
-    list->tail = ne;
-  } else {
-    ne->next = e;
-    ne->prev = e->prev;
-    if (e->prev)
-      e->prev->next = ne;
-    else
-      list->head = ne;
-    e->prev = ne;
-  }
-
-  ++list->size;
-
-  return 1;
-}
-#endif
-
-int 
-Curl_llist_remove(curl_llist *list, curl_llist_element *e, void *user)
+int
+Curl_llist_remove(struct curl_llist *list, struct curl_llist_element *e,
+                  void *user)
 {
   if (e == NULL || list->size == 0)
     return 1;
@@ -147,28 +120,8 @@ Curl_llist_remove(curl_llist *list, curl_llist_element *e, void *user)
   return 1;
 }
 
-#if 0
-int 
-Curl_llist_remove_next(curl_llist *list, curl_llist_element *e, void *user)
-{
-  return Curl_llist_remove(list, e->next, user);
-}
-
-int 
-Curl_llist_remove_prev(curl_llist *list, curl_llist_element *e, void *user)
-{
-  return Curl_llist_remove(list, e->prev, user);
-}
-
-size_t 
-Curl_llist_count(curl_llist *list)
-{
-  return list->size;
-}
-#endif
-
-void 
-Curl_llist_destroy(curl_llist *list, void *user)
+void
+Curl_llist_destroy(struct curl_llist *list, void *user)
 {
   if(list) {
     while (list->size > 0)
@@ -177,3 +130,9 @@ Curl_llist_destroy(curl_llist *list, void *user)
     free(list);
   }
 }
+
+size_t
+Curl_llist_count(struct curl_llist *list)
+{
+  return list->size;
+}
