@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2005, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: multiif.h,v 1.4 2006-04-10 15:00:54 bagder Exp $
+ * $Id: multiif.h,v 1.7 2007-04-10 20:46:40 bagder Exp $
  ***************************************************************************/
 
 /*
@@ -30,6 +30,9 @@ void Curl_expire(struct SessionHandle *data, long milli);
 
 void Curl_multi_rmeasy(void *multi, CURL *data);
 
+bool Curl_multi_canPipeline(struct Curl_multi* multi);
+void Curl_multi_handlePipeBreak(struct SessionHandle *data);
+
 /* the write bits start at bit 16 for the *getsock() bitmap */
 #define GETSOCK_WRITEBITSTART 16
 
