@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: telnet.c,v 1.78 2006-07-19 21:14:02 yangtse Exp $
+ * $Id: telnet.c,v 1.90 2007-03-25 02:30:58 yangtse Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -30,12 +30,6 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
 
 #if defined(WIN32)
 #include <time.h>
@@ -45,7 +39,9 @@
 #include <sys/socket.h>
 #endif
 #include <netinet/in.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -63,8 +59,7 @@
 #include <sys/param.h>
 #endif
 
-
-#endif
+#endif  /* WIN32 */
 
 #include "urldata.h"
 #include <curl/curl.h>
@@ -100,7 +95,11 @@
 #define  CURL_SB_EOF(x) (x->subpointer >= x->subend)
 #define  CURL_SB_LEN(x) (x->subend - x->subpointer)
 
-#ifdef WIN32
+#ifdef CURL_DISABLE_VERBOSE_STRINGS
+#define printoption(a,b,c,d)  do { } while (0)
+#endif
+
+#ifdef USE_WINSOCK
 typedef FARPROC WSOCK2_FUNC;
 static CURLcode check_wsock2 ( struct SessionHandle *data );
 #endif
@@ -110,9 +109,11 @@ void telrcv(struct connectdata *,
             unsigned char *inbuf,       /* Data received from socket */
             ssize_t count);             /* Number of bytes received */
 
+#ifndef CURL_DISABLE_VERBOSE_STRINGS
 static void printoption(struct SessionHandle *data,
                         const char *direction,
                         int cmd, int option);
+#endif
 
 static void negotiate(struct connectdata *);
 static void send_negotiation(struct connectdata *, int cmd, int option);
@@ -169,7 +170,7 @@ struct TELNET {
   TelnetReceive telrcv_state;
 };
 
-#ifdef WIN32
+#ifdef USE_WINSOCK
 static CURLcode
 check_wsock2 ( struct SessionHandle *data )
 {
@@ -177,7 +178,7 @@ check_wsock2 ( struct SessionHandle *data )
   WORD wVersionRequested;
   WSADATA wsaData;
 
-  curlassert(data);
+  DEBUGASSERT(data);
 
   /* telnet requires at least WinSock 2.0 so ask for it. */
   wVersionRequested = MAKEWORD(2, 0);
@@ -208,6 +209,7 @@ check_wsock2 ( struct SessionHandle *data )
   return CURLE_OK;
 }
 #endif
+
 static
 CURLcode init_telnet(struct connectdata *conn)
 {
@@ -217,7 +219,7 @@ CURLcode init_telnet(struct connectdata *conn)
   if(!tn)
     return CURLE_OUT_OF_MEMORY;
 
-  conn->proto.telnet = (void *)tn; /* make us known */
+  conn->data->reqdata.proto.telnet = (void *)tn; /* make us known */
 
   tn->telrcv_state = CURL_TS_DATA;
 
@@ -236,7 +238,7 @@ CURLcode init_telnet(struct connectdata *conn)
 static void negotiate(struct connectdata *conn)
 {
   int i;
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *) conn->data->reqdata.proto.telnet;
 
   for(i = 0;i < CURL_NTELOPTS;i++)
   {
@@ -248,6 +250,7 @@ static void negotiate(struct connectdata *conn)
   }
 }
 
+#ifndef CURL_DISABLE_VERBOSE_STRINGS
 static void printoption(struct SessionHandle *data,
                         const char *direction, int cmd, int option)
 {
@@ -286,6 +289,7 @@ static void printoption(struct SessionHandle *data,
     }
   }
 }
+#endif
 
 static void send_negotiation(struct connectdata *conn, int cmd, int option)
 {
@@ -300,7 +304,7 @@ static void send_negotiation(struct connectdata *conn, int cmd, int option)
 
    bytes_written = swrite(conn->sock[FIRSTSOCKET], buf, 3);
    if(bytes_written < 0) {
-     err = Curl_sockerrno();
+     err = SOCKERRNO;
      failf(data,"Sending data failed (%d)",err);
    }
 
@@ -310,7 +314,7 @@ static void send_negotiation(struct connectdata *conn, int cmd, int option)
 static
 void set_remote_option(struct connectdata *conn, int option, int newstate)
 {
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
   if(newstate == CURL_YES)
   {
     switch(tn->him[option])
@@ -392,7 +396,7 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
 static
 void rec_will(struct connectdata *conn, int option)
 {
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
   switch(tn->him[option])
   {
     case CURL_NO:
@@ -445,7 +449,7 @@ void rec_will(struct connectdata *conn, int option)
 static
 void rec_wont(struct connectdata *conn, int option)
 {
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
   switch(tn->him[option])
   {
     case CURL_NO:
@@ -490,7 +494,7 @@ void rec_wont(struct connectdata *conn, int option)
 static void
 set_local_option(struct connectdata *conn, int option, int newstate)
 {
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
   if(newstate == CURL_YES)
   {
     switch(tn->us[option])
@@ -572,7 +576,7 @@ set_local_option(struct connectdata *conn, int option, int newstate)
 static
 void rec_do(struct connectdata *conn, int option)
 {
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
   switch(tn->us[option])
   {
     case CURL_NO:
@@ -625,7 +629,7 @@ void rec_do(struct connectdata *conn, int option)
 static
 void rec_dont(struct connectdata *conn, int option)
 {
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
   switch(tn->us[option])
   {
     case CURL_NO:
@@ -787,7 +791,7 @@ static CURLcode check_telnet_options(struct connectdata *conn)
   char option_arg[256];
   char *buf;
   struct SessionHandle *data = conn->data;
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
 
   /* Add the user name as an environment variable if it
      was given on the command line */
@@ -858,7 +862,7 @@ static void suboption(struct connectdata *conn)
   char varname[128];
   char varval[128];
   struct SessionHandle *data = conn->data;
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)data->reqdata.proto.telnet;
 
   printsub(data, '<', (unsigned char *)tn->subbuffer, CURL_SB_LEN(tn)+2);
   switch (CURL_SB_GET(tn)) {
@@ -869,7 +873,7 @@ static void suboption(struct connectdata *conn)
                CURL_TELQUAL_IS, tn->subopt_ttype, CURL_IAC, CURL_SE);
       bytes_written = swrite(conn->sock[FIRSTSOCKET], temp, len);
       if(bytes_written < 0) {
-        err = Curl_sockerrno();
+        err = SOCKERRNO;
         failf(data,"Sending data failed (%d)",err);
       }
       printsub(data, '>', &temp[2], len-2);
@@ -881,7 +885,7 @@ static void suboption(struct connectdata *conn)
                CURL_TELQUAL_IS, tn->subopt_xdisploc, CURL_IAC, CURL_SE);
       bytes_written = swrite(conn->sock[FIRSTSOCKET], temp, len);
       if(bytes_written < 0) {
-        err = Curl_sockerrno();
+        err = SOCKERRNO;
         failf(data,"Sending data failed (%d)",err);
       }
       printsub(data, '>', &temp[2], len-2);
@@ -908,7 +912,7 @@ static void suboption(struct connectdata *conn)
       len += 2;
       bytes_written = swrite(conn->sock[FIRSTSOCKET], temp, len);
       if(bytes_written < 0) {
-        err = Curl_sockerrno();
+        err = SOCKERRNO;
         failf(data,"Sending data failed (%d)",err);
       }
       printsub(data, '>', &temp[2], len-2);
@@ -925,7 +929,7 @@ void telrcv(struct connectdata *conn,
   unsigned char c;
   int in = 0;
   struct SessionHandle *data = conn->data;
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)data->reqdata.proto.telnet;
 
   while(count--)
   {
@@ -940,7 +944,7 @@ void telrcv(struct connectdata *conn,
           break;   /* Ignore \0 after CR */
         }
 
-        Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
+        Curl_client_write(conn, CLIENTWRITE_BODY, (char *)&c, 1);
         continue;
 
       case CURL_TS_DATA:
@@ -954,7 +958,7 @@ void telrcv(struct connectdata *conn,
           tn->telrcv_state = CURL_TS_CR;
         }
 
-        Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
+        Curl_client_write(conn, CLIENTWRITE_BODY, (char *)&c, 1);
         continue;
 
       case CURL_TS_IAC:
@@ -978,7 +982,7 @@ void telrcv(struct connectdata *conn,
           tn->telrcv_state = CURL_TS_SB;
           continue;
         case CURL_IAC:
-          Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
+          Curl_client_write(conn, CLIENTWRITE_BODY, (char *)&c, 1);
           break;
         case CURL_DM:
         case CURL_NOP:
@@ -1070,15 +1074,16 @@ void telrcv(struct connectdata *conn,
   }
 }
 
-CURLcode Curl_telnet_done(struct connectdata *conn, CURLcode status)
+CURLcode Curl_telnet_done(struct connectdata *conn, CURLcode status, bool premature)
 {
-  struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  struct TELNET *tn = (struct TELNET *)conn->data->reqdata.proto.telnet;
   (void)status; /* unused */
+  (void)premature; /* not used */
 
   curl_slist_free_all(tn->telnet_vars);
 
-  free(conn->proto.telnet);
-  conn->proto.telnet = NULL;
+  free(conn->data->reqdata.proto.telnet);
+  conn->data->reqdata.proto.telnet = NULL;
 
   return CURLE_OK;
 }
@@ -1088,7 +1093,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   CURLcode code;
   struct SessionHandle *data = conn->data;
   curl_socket_t sockfd = conn->sock[FIRSTSOCKET];
-#ifdef WIN32
+#ifdef USE_WINSOCK
   HMODULE wsock2;
   WSOCK2_FUNC close_event_func;
   WSOCK2_FUNC create_event_func;
@@ -1117,13 +1122,13 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   if(code)
     return code;
 
-  tn = (struct TELNET *)conn->proto.telnet;
+  tn = (struct TELNET *)data->reqdata.proto.telnet;
 
   code = check_telnet_options(conn);
   if(code)
     return code;
 
-#ifdef WIN32
+#ifdef USE_WINSOCK
   /*
   ** This functionality only works with WinSock >= 2.0.  So,
   ** make sure have it.
@@ -1136,7 +1141,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   /* load ws2_32.dll and get the function pointers we need. */
   wsock2 = LoadLibrary("WS2_32.DLL");
   if (wsock2 == NULL) {
-    failf(data,"failed to load WS2_32.DLL (%d)",GetLastError());
+    failf(data,"failed to load WS2_32.DLL (%d)", ERRNO);
     return CURLE_FAILED_INIT;
   }
 
@@ -1144,16 +1149,16 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   create_event_func = GetProcAddress(wsock2,"WSACreateEvent");
   if (create_event_func == NULL) {
     failf(data,"failed to find WSACreateEvent function (%d)",
-          GetLastError());
+          ERRNO);
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
 
   /* And WSACloseEvent */
   close_event_func = GetProcAddress(wsock2,"WSACloseEvent");
-  if (create_event_func == NULL) {
+  if (close_event_func == NULL) {
     failf(data,"failed to find WSACloseEvent function (%d)",
-          GetLastError());
+          ERRNO);
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1162,7 +1167,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   event_select_func = GetProcAddress(wsock2,"WSAEventSelect");
   if (event_select_func == NULL) {
     failf(data,"failed to find WSAEventSelect function (%d)",
-          GetLastError());
+          ERRNO);
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1171,7 +1176,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   enum_netevents_func = GetProcAddress(wsock2,"WSAEnumNetworkEvents");
   if (enum_netevents_func == NULL) {
     failf(data,"failed to find WSAEnumNetworkEvents function (%d)",
-          GetLastError());
+          ERRNO);
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1184,7 +1189,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   /* First, create a sockets event object */
   event_handle = (WSAEVENT)create_event_func();
   if (event_handle == WSA_INVALID_EVENT) {
-    failf(data,"WSACreateEvent failed (%d)",WSAGetLastError());
+    failf(data,"WSACreateEvent failed (%d)", SOCKERRNO);
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1312,7 +1317,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
 
   /* We called WSACreateEvent, so call WSACloseEvent */
   if (close_event_func(event_handle) == FALSE) {
-    infof(data,"WSACloseEvent failed (%d)",WSAGetLastError());
+    infof(data,"WSACloseEvent failed (%d)", SOCKERRNO);
   }
 
   /* "Forget" pointers into the library we're about to free */
@@ -1323,7 +1328,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
 
   /* We called LoadLibrary, so call FreeLibrary */
   if (!FreeLibrary(wsock2))
-    infof(data,"FreeLibrary(wsock2) failed (%d)",GetLastError());
+    infof(data,"FreeLibrary(wsock2) failed (%d)", ERRNO);
 #else
   pfd[0].fd = sockfd;
   pfd[0].events = POLLIN;
@@ -1383,7 +1388,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
     if(data->set.timeout) {
       struct timeval now;           /* current time */
       now = Curl_tvnow();
-      if(Curl_tvdiff(now, conn->created)/1000 >= data->set.timeout) {
+      if(Curl_tvdiff(now, conn->created) >= data->set.timeout) {
         failf(data, "Time-out");
         code = CURLE_OPERATION_TIMEOUTED;
         keepon = FALSE;
@@ -1392,7 +1397,7 @@ CURLcode Curl_telnet(struct connectdata *conn, bool *done)
   }
 #endif
   /* mark this as "no further transfer wanted" */
-  Curl_Transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
+  Curl_setup_transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
 
   return code;
 }
