@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 #***************************************************************************
 #                                  _   _ ____  _
 #  Project                     ___| | | |  _ \| |
@@ -6,7 +6,7 @@
 #                            | (__| |_| |  _ <| |___
 #                             \___|\___/|_| \_\_____|
 #
-# Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
+# Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: ftpserver.pl,v 1.43 2004/05/17 08:02:23 bagder Exp $
+# $Id: ftpserver.pl,v 1.79 2007-03-01 16:42:02 yangtse Exp $
 ###########################################################################
 
 # This is the FTP server designed for the curl test suite.
@@ -30,31 +30,37 @@
 # You may optionally specify port on the command line, otherwise it'll
 # default to port 8921.
 #
-
-use Socket;
-use FileHandle;
+# All socket/network/TCP related stuff is done by the 'sockfilt' program.
+#
 
 use strict;
+use IPC::Open2;
+#use Time::HiRes qw( gettimeofday ); # not available in perl 5.6
 
 require "getpart.pm";
+require "ftp.pm";
+
+
+my $ftpdnum="";
 
 # open and close each time to allow removal at any time
 sub logmsg {
+ # if later than perl 5.6 is used
+ #   my ($seconds, $microseconds) = gettimeofday;
+    my $seconds = time();
     my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
-        localtime(time);
-    open(FTPLOG, ">>log/ftpd.log");
-    printf FTPLOG ("%02d:%02d:%02d (%d) ",
-                   $hour, $min, $sec, $$);
+        localtime($seconds);
+    open(FTPLOG, ">>log/ftpd$ftpdnum.log");
+    printf FTPLOG ("%02d:%02d:%02d ", $hour, $min, $sec);
     print FTPLOG @_;
     close(FTPLOG);
 }
 
 sub ftpmsg {
   # append to the server.input file
-  open(INPUT, ">>log/server.input") ||
-    logmsg "failed to open log/server.input\n";
+  open(INPUT, ">>log/server$ftpdnum.input") ||
+    logmsg "failed to open log/server$ftpdnum.input\n";
 
-  INPUT->autoflush(1);
   print INPUT @_;
   close(INPUT);
 
@@ -64,11 +70,20 @@ sub ftpmsg {
 }
 
 my $verbose=0; # set to 1 for debugging
+my $pasvbadip=0;
 my $retrweirdo=0;
 my $retrnosize=0;
 my $srcdir=".";
-
+my $nosave=0;
+my $controldelay=0; # set to 1 to delay the control connect data sending to
+ # test that curl deals with that nicely
+my $slavepid; # for the DATA connection sockfilt slave process
+my $ipv6;
+my $ext; # append to log/pid file names
+my $grok_eprt;
 my $port = 8921; # just a default
+my $pidfile = ".ftpd.pid"; # a default, use --pidfile
+
 do {
     if($ARGV[0] eq "-v") {
         $verbose=1;
@@ -77,41 +92,149 @@ do {
         $srcdir=$ARGV[1];
         shift @ARGV;
     }
-    elsif($ARGV[0] =~ /^(\d+)$/) {
-        $port = $1;
+    elsif($ARGV[0] eq "--id") {
+        $ftpdnum=$ARGV[1];
+        shift @ARGV;
+    }
+    elsif($ARGV[0] eq "--pidfile") {
+        $pidfile=$ARGV[1];
+        shift @ARGV;
+    }
+    elsif($ARGV[0] eq "--ipv6") {
+        $ipv6="--ipv6";
+        $ext="ipv6";
+        $grok_eprt = 1;
+    }
+    elsif($ARGV[0] eq "--port") {
+        $port = $ARGV[1];
+        shift @ARGV;
     }
 } while(shift @ARGV);
 
-my $proto = getprotobyname('tcp') || 6;
+sub catch_zap {
+    my $signame = shift;
+    print STDERR "ftpserver.pl received SIG$signame, exiting\n";
+    ftpkillslaves(1);
+    die "Somebody sent me a SIG$signame";
+}
+$SIG{INT} = \&catch_zap;
+$SIG{KILL} = \&catch_zap;
 
-socket(Server, PF_INET, SOCK_STREAM, $proto)|| die "socket: $!";
-setsockopt(Server, SOL_SOCKET, SO_REUSEADDR,
-           pack("l", 1)) || die "setsockopt: $!";
-bind(Server, sockaddr_in($port, INADDR_ANY))|| die "bind: $!";
-listen(Server,SOMAXCONN) || die "listen: $!";
+my $sfpid;
 
-#print "FTP server started on port $port\n";
+local(*SFREAD, *SFWRITE);
 
-open(PID, ">.ftp.pid");
-print PID $$;
+sub sysread_or_die {
+    my $FH     = shift;
+    my $scalar = shift;
+    my $length = shift;
+    my $fcaller;
+    my $lcaller;
+    my $result;
+
+    $result = sysread($$FH, $$scalar, $length);
+
+    if(not defined $result) {
+        ($fcaller, $lcaller) = (caller)[1,2];
+        logmsg "Failed to read input\n";
+        logmsg "Error: ftp$ftpdnum$ext sysread error: $!\n";
+        kill(9, $sfpid);
+        die "Died in sysread_or_die() when called from $fcaller " .
+            "at line $lcaller. ftp$ftpdnum$ext sysread error: $!\n";
+    }
+    elsif($result == 0) {
+        ($fcaller, $lcaller) = (caller)[1,2];
+        logmsg "Failed to read input\n";
+        logmsg "Error: ftp$ftpdnum$ext read zero\n";
+        kill(9, $sfpid);
+        die "Died in sysread_or_die() when called from $fcaller " .
+            "at line $lcaller. ftp$ftpdnum$ext read zero\n";
+    }
+
+    return $result;
+}
+
+sub startsf {
+    my $cmd="./server/sockfilt --port $port --logfile log/sockctrl$ftpdnum$ext.log --pidfile .sockfilt$ftpdnum$ext.pid $ipv6";
+    $sfpid = open2(*SFREAD, *SFWRITE, $cmd);
+
+    print STDERR "$cmd\n" if($verbose);
+
+    print SFWRITE "PING\n";
+    my $pong;
+    sysread SFREAD, $pong, 5;
+
+    if($pong !~ /^PONG/) {
+        logmsg "Failed sockfilt command: $cmd\n";
+        kill(9, $sfpid);
+        die "Failed to start sockfilt!";
+    }
+}
+
+# remove the file here so that if startsf() fails, it is very noticable 
+unlink($pidfile);
+
+startsf();
+
+logmsg sprintf("FTP server listens on port IPv%d/$port\n", $ipv6?6:4);
+open(PID, ">$pidfile");
+print PID $$."\n";
 close(PID);
 
-my $waitedpid = 0;
-my $paddr;
+logmsg("logged pid $$ in $pidfile\n");
 
-sub REAPER {
-    $waitedpid = wait;
-    $SIG{CHLD} = \&REAPER;  # loathe sysV
-    logmsg "reaped $waitedpid" . ($? ? " with exit $?\n" : "\n");
+sub sockfilt {
+    my $l;
+    foreach $l (@_) {
+        printf SFWRITE "DATA\n%04x\n", length($l);
+        print SFWRITE $l;
+    }
+}
+
+
+# Send data to the client on the control stream, which happens to be plain
+# stdout.
+
+sub sendcontrol {
+    if(!$controldelay) {
+        # spit it all out at once
+        sockfilt @_;
+    }
+    else {
+        my $a = join("", @_);
+        my @a = split("", $a);
+
+        for(@a) {
+            sockfilt $_;
+            select(undef, undef, undef, 0.01);
+        }
+    }
+    my $log;
+    foreach $log (@_) {
+        my $l = $log;
+        $l =~ s/[\r\n]//g;
+        logmsg "> \"$l\"\n";
+    }
+}
+
+# Send data to the client on the data stream
+
+sub senddata {
+    my $l;
+    foreach $l (@_) {
+        printf DWRITE "DATA\n%04x\n", length($l);
+        print DWRITE $l;
+    }
 }
 
 # USER is ok in fresh state
 my %commandok = (
-                 'USER' => 'fresh',
+                 'USER' => 'fresh|passwd',
                  'PASS' => 'passwd',
                  'PASV' => 'loggedin|twosock',
                  'EPSV' => 'loggedin|twosock',
                  'PORT' => 'loggedin|twosock',
+                 'EPRT' => 'loggedin|twosock',
                  'TYPE' => 'loggedin|twosock',
                  'LIST' => 'twosock',
                  'NLST' => 'twosock',
@@ -119,6 +242,7 @@ my %commandok = (
                  'STOR' => 'twosock',
                  'APPE' => 'twosock',
                  'REST' => 'twosock',
+                 'ACCT' => 'loggedin',
                  'CWD'  => 'loggedin|twosock',
                  'SYST' => 'loggedin',
                  'SIZE' => 'loggedin|twosock',
@@ -129,12 +253,14 @@ my %commandok = (
                  'RNTO'  => 'loggedin|twosock',
                  'DELE' => 'loggedin|twosock',
                  'MDTM' => 'loggedin|twosock',
+                 'NOOP' => 'loggedin|twosock',
                  );
 
 # initially, we're in 'fresh' state
 my %statechange = ( 'USER' => 'passwd',    # USER goes to passwd state
                     'PASS' => 'loggedin',  # PASS goes to loggedin state
                     'PORT' => 'twosock',   # PORT goes to twosock
+                    'EPRT' => 'twosock',   # EPRT goes to twosock
                     'PASV' => 'twosock',   # PASV goes to twosock
                     'EPSV' => 'twosock',   # EPSV goes to twosock
                     );
@@ -155,10 +281,12 @@ my %displaytext = ('USER' => '331 We are happy you popped in!',
                    'DELE' => '200 OK OK OK whatever you say',
                    'RNFR' => '350 Received your order. Please provide more',
                    'RNTO' => '250 Ok, thanks. File renaming completed.',
+                   'NOOP' => '200 Yes, I\'m very good at doing nothing.',
                    );
 
 # callback functions for certain commands
 my %commandfunc = ( 'PORT' => \&PORT_command,
+                    'EPRT' => \&PORT_command,
                     'LIST' => \&LIST_command,
                     'NLST' => \&NLST_command,
                     'PASV' => \&PASV_command,
@@ -173,8 +301,23 @@ my %commandfunc = ( 'PORT' => \&PORT_command,
 
 
 sub close_dataconn {
-    close(SOCK);
-    logmsg "Closed data connection\n";
+    my ($closed)=@_; # non-zero if already disconnected
+
+    if(!$closed) {
+        logmsg "* disconnect data connection\n";
+        print DWRITE "DISC\n";
+        my $i;
+        sysread DREAD, $i, 5;
+    }
+    else {
+        logmsg "data connection already disconnected\n";
+    }
+    logmsg "=====> Closed data connection\n";
+
+    logmsg "* quit sockfilt for data (pid $slavepid)\n";
+    print DWRITE "QUIT\n";
+    waitpid $slavepid, 0;
+    $slavepid=0;
 }
 
 my $rest=0;
@@ -201,12 +344,10 @@ my @ftpdir=("total 20\r\n",
 
     logmsg "pass LIST data on data connection\n";
     for(@ftpdir) {
-        print SOCK $_;
+        senddata $_;
     }
-    close_dataconn();
-    logmsg "done passing data\n";
-
-    print "226 ASCII transfer complete\r\n";
+    close_dataconn(0);
+    sendcontrol "226 ASCII transfer complete\r\n";
     return 0;
 }
 
@@ -214,10 +355,10 @@ sub NLST_command {
     my @ftpdir=("file", "with space", "fake", "..", " ..", "funny", "README");
     logmsg "pass NLST data on data connection\n";
     for(@ftpdir) {
-        print SOCK "$_\r\n";
+        senddata "$_\r\n";
     }
-    close_dataconn();
-    print "226 ASCII transfer complete\r\n";
+    close_dataconn(0);
+    sendcontrol "226 ASCII transfer complete\r\n";
     return 0;
 }
 
@@ -226,24 +367,19 @@ sub MDTM_command {
 
     loadtest("$srcdir/data/test$testno");
 
-    logmsg "MDTM $testno\n";
-
     my @data = getpart("reply", "mdtm");
 
     my $reply = $data[0];
     chomp $reply;
 
     if($reply <0) {
-        print "550 $testno: no such file.\r\n";
-        logmsg "MDTM $testno: no such file\n";
+        sendcontrol "550 $testno: no such file.\r\n";
     }
     elsif($reply) {
-        print "$reply\r\n";
-        logmsg "MDTM $testno returned $reply\n";
+        sendcontrol "$reply\r\n";
     }
     else {
-        print "500 MDTM: no such command.\r\n";
-        logmsg "MDTM: no such command\n";
+        sendcontrol "500 MDTM: no such command.\r\n";
     }
     return 0;
 }
@@ -253,7 +389,12 @@ sub SIZE_command {
 
     loadtest("$srcdir/data/test$testno");
 
-    logmsg "SIZE file \"$testno\"\n";
+    if($testno eq "verifiedserver") {
+        my $response = "WE ROOLZ: $$\r\n";
+        my $size = length($response);
+        sendcontrol "213 $size\r\n";
+        return 0;
+    }
 
     my @data = getpart("reply", "size");
 
@@ -261,12 +402,10 @@ sub SIZE_command {
 
     if($size) {
         if($size > -1) {
-            print "213 $size\r\n";
-            logmsg "SIZE $testno returned $size\n";
+            sendcontrol "213 $size\r\n";
         }
         else {
-            print "550 $testno: No such file or directory.\r\n";
-            logmsg "SIZE $testno: no such file\n";
+            sendcontrol "550 $testno: No such file or directory.\r\n";
         }
     }
     else {
@@ -276,39 +415,35 @@ sub SIZE_command {
             $size += length($_);
         }
         if($size) {
-            print "213 $size\r\n";
-            logmsg "SIZE $testno returned $size\n";
+            sendcontrol "213 $size\r\n";
         }
         else {
-            print "550 $testno: No such file or directory.\r\n";
-            logmsg "SIZE $testno: no such file\n";
+            sendcontrol "550 $testno: No such file or directory.\r\n";
         }
     }
     return 0;
 }
 
 sub RETR_command {
-    my $testno = $_[0];
-
-    logmsg "RETR file \"$testno\"\n";
+    my ($testno) = @_;
 
     if($testno =~ /^verifiedserver$/) {
         # this is the secret command that verifies that this actually is
         # the curl test server
         my $response = "WE ROOLZ: $$\r\n";
         my $len = length($response);
-        print "150 Binary junk ($len bytes).\r\n";
-        logmsg "pass our pid on the data connection\n";
-        print SOCK "WE ROOLZ: $$\r\n";
-        close_dataconn();
-        print "226 File transfer complete\r\n";
+        sendcontrol "150 Binary junk ($len bytes).\r\n";
+        senddata "WE ROOLZ: $$\r\n";
+        close_dataconn(0);
+        sendcontrol "226 File transfer complete\r\n";
         if($verbose) {
             print STDERR "FTPD: We returned proof we are the test server\n";
         }
-        logmsg "we returned proof that we are the test server\n";
         return 0;
     }
 
+    $testno =~ s/^([^0-9]*)//;
+
     loadtest("$srcdir/data/test$testno");
 
     my @data = getpart("reply", "data");
@@ -318,7 +453,9 @@ sub RETR_command {
         $size += length($_);
     }
 
-    if($size) {
+    my %hash = getpartattr("reply", "data");
+
+    if($size || $hash{'sendzero'}) {
     
         if($rest) {
             # move read pointer forward
@@ -327,16 +464,14 @@ sub RETR_command {
             $rest = 0; # reset REST offset again
         }
         if($retrweirdo) {
-            print "150 Binary data connection for $testno () ($size bytes).\r\n",
+            sendcontrol "150 Binary data connection for $testno () ($size bytes).\r\n",
             "226 File transfer complete\r\n";
-            logmsg "150+226 in one shot!\n";
 
-            logmsg "pass RETR data on data connection\n";
             for(@data) {
                 my $send = $_;
-                print SOCK $send;
+                senddata $send;
             }
-            close_dataconn();
+            close_dataconn(0);
             $retrweirdo=0; # switch off the weirdo again!
         }
         else {
@@ -345,21 +480,18 @@ sub RETR_command {
                 $sz = "size?";
             }
 
-            print "150 Binary data connection for $testno () $sz.\r\n";
-            logmsg "150 Binary data connection for $testno () $sz.\n";
+            sendcontrol "150 Binary data connection for $testno () $sz.\r\n";
 
-            logmsg "pass RETR data on data connection\n";
             for(@data) {
                 my $send = $_;
-                print SOCK $send;
+                senddata $send;
             }
-            close_dataconn();
-            print "226 File transfer complete\r\n";
+            close_dataconn(0);
+            sendcontrol "226 File transfer complete\r\n";
         }
     }
     else {
-        print "550 $testno: No such file or directory.\r\n";
-        logmsg "550 $testno: no such file\n";
+        sendcontrol "550 $testno: No such file or directory.\r\n";
     }
     return 0;
 }
@@ -371,132 +503,216 @@ sub STOR_command {
 
     logmsg "STOR test number $testno in $filename\n";
 
-    print "125 Gimme gimme gimme!\r\n";
-
-    logmsg "retrieve STOR data on data connection\n";
+    sendcontrol "125 Gimme gimme gimme!\r\n";
 
     open(FILE, ">$filename") ||
         return 0; # failed to open output
 
     my $line;
     my $ulsize=0;
-    while (defined($line = <SOCK>)) {
-        $ulsize += length($line);
-        print FILE $line;
+    my $disc=0;
+    while (5 == (sysread DREAD, $line, 5)) {
+        if($line eq "DATA\n") {
+            my $i;
+            sysread DREAD, $i, 5;
+
+            #print STDERR "  GOT: $i";
+
+            my $size = hex($i);
+            sysread DREAD, $line, $size;
+            
+            #print STDERR "  GOT: $size bytes\n";
+
+            $ulsize += $size;
+            print FILE $line if(!$nosave);
+            logmsg "> Appending $size bytes to file\n";
+        }
+        elsif($line eq "DISC\n") {
+            # disconnect!
+            $disc=1;
+            last;
+        }
+        else {
+            logmsg "No support for: $line";
+            last;
+        }
+    }
+    if($nosave) {
+        print FILE "$ulsize bytes would've been stored here\n";
     }
     close(FILE);
-    close_dataconn();
-
+    close_dataconn($disc);
     logmsg "received $ulsize bytes upload\n";
-
-    print "226 File transfer complete\r\n";
+    sendcontrol "226 File transfer complete\r\n";
     return 0;
 }
 
-my $pasvport=9000;
 sub PASV_command {
     my ($arg, $cmd)=@_;
+    my $pasvport;
+    my $pidf=".sockdata$ftpdnum$ext.pid";
 
-    socket(Server2, PF_INET, SOCK_STREAM, $proto) || die "socket: $!";
-    setsockopt(Server2, SOL_SOCKET, SO_REUSEADDR,
-               pack("l", 1)) || die "setsockopt: $!";
-
-    my $ok=0;
-
-    $pasvport++; # don't reuse the previous
-    for(1 .. 10) {
-        if($pasvport > 65535) {
-            $pasvport = 1025;
-        }
-        if(bind(Server2, sockaddr_in($pasvport, INADDR_ANY))) {
-            $ok=1;
-            last;
-        }
-        $pasvport+= 3; # try another port please
+    my $prev = checkserver($pidf);
+    if($prev > 0) {
+        print "kill existing server: $prev\n" if($verbose);
+        kill(9, $prev);
     }
-    if(!$ok) {
-        print "500 no free ports!\r\n";
-        logmsg "couldn't find free port\n";
+
+    # We fire up a new sockfilt to do the data tranfer for us.
+    $slavepid = open2(\*DREAD, \*DWRITE,
+                      "./server/sockfilt --port 0 --logfile log/sockdata$ftpdnum$ext.log --pidfile $pidf $ipv6");
+
+    print DWRITE "PING\n";
+    my $pong;
+
+    sysread_or_die(\*DREAD, \$pong, 5);
+
+    if($pong !~ /^PONG/) {
+        kill(9, $slavepid);
+        sendcontrol "500 no free ports!\r\n";
+        logmsg "failed to run sockfilt for data connection\n";
         return 0;
     }
-    listen(Server2,SOMAXCONN) || die "listen: $!";
+
+    logmsg "Run sockfilt for data on pid $slavepid\n";
+
+    # Find out what port we listen on
+    my $i;
+    print DWRITE "PORT\n";
+        
+    # READ the response code
+    sysread_or_die(\*DREAD, \$i, 5);
+
+    # READ the response size
+    sysread_or_die(\*DREAD, \$i, 5);
+
+    my $size = hex($i);
+        
+    # READ the response data
+    sysread_or_die(\*DREAD, \$i, $size);
+        
+    # The data is in the format
+    # IPvX/NNN
+
+    if($i =~ /IPv(\d)\/(\d+)/) {
+        # FIX: deal with IP protocol version
+        $pasvport = $2;
+    }
 
     if($cmd ne "EPSV") {
         # PASV reply
-        logmsg "replying to a $cmd command\n";
-        printf("227 Entering Passive Mode (127,0,0,1,%d,%d)\n",
-               ($pasvport/256), ($pasvport%256));
+        my $p="127,0,0,1";
+        if($pasvbadip) {
+            $p="1,2,3,4";
+        }
+        sendcontrol sprintf("227 Entering Passive Mode ($p,%d,%d)\n",
+                            ($pasvport/256), ($pasvport%256));
     }
     else {
         # EPSV reply
-        logmsg "replying to a $cmd command\n";
-        printf("229 Entering Passive Mode (|||%d|)\n", $pasvport);
+        sendcontrol sprintf("229 Entering Passive Mode (|||%d|)\n", $pasvport);
     }
 
-
-    my $paddr;
     eval {
         local $SIG{ALRM} = sub { die "alarm\n" };
-        alarm 2; # assume swift operations!
-        $paddr = accept(SOCK, Server2);
+
+	# assume swift operations unless explicitly slow
+	alarm ($controldelay?20:5);
+
+        # Wait for 'CNCT'
+	my $input;
+
+        while(sysread(DREAD, $input, 5)) {
+
+	    if($input !~ /^CNCT/) {
+		# we wait for a connected client
+		logmsg "Odd, we got $input from client\n";
+		next;
+	    }
+	    logmsg "====> Client DATA connect\n";
+	    last;
+	}
         alarm 0;
     };
     if ($@) {
         # timed out
-        
-        close(Server2);
+
+        print DWRITE "QUIT\n";
+        waitpid $slavepid, 0;
         logmsg "accept failed\n";
+        $slavepid=0;
         return;
     }
     else {
-        logmsg "accept worked\n";
-
-        my($iport,$iaddr) = sockaddr_in($paddr);
-        my $name = gethostbyaddr($iaddr,AF_INET);
-
-        close(Server2); # close the listener when its served its purpose!
-
-        logmsg "data connection from $name [", inet_ntoa($iaddr),
-        "] at port $iport\n";
+        logmsg "data connection setup on port $pasvport\n";
     }
 
     return;
 }
 
+# Support both PORT and EPRT here. Consider LPRT too.
 
 sub PORT_command {
-    my $arg = $_[0];
+    my ($arg, $cmd) = @_;
+    my $port;
 
-    if($arg !~ /(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)/) {
-        logmsg "bad PORT-line: $arg\n";
-        print "500 silly you, go away\r\n";
+    # We always ignore the given IP and use localhost.
+
+    if($cmd eq "PORT") {
+        if($arg !~ /(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)/) {
+            logmsg "bad PORT-line: $arg\n";
+            sendcontrol "500 silly you, go away\r\n";
+            return 0;
+        }
+        $port = ($5<<8)+$6;
+    }
+    # EPRT |2|::1|49706|
+    elsif(($cmd eq "EPRT") && ($grok_eprt)) {
+        if($arg !~ /(\d+)\|([^\|]+)\|(\d+)/) {
+            sendcontrol "500 silly you, go away\r\n";
+            return 0;
+        }
+        sendcontrol "200 Thanks for dropping by. We contact you later\r\n";
+        $port = $3;
+    }
+    else {
+        sendcontrol "500 we don't like $cmd now\r\n";
         return 0;
     }
-    my $iaddr = inet_aton("$1.$2.$3.$4");
-
-    my $port = ($5<<8)+$6;
 
     if(!$port || $port > 65535) {
         print STDERR "very illegal PORT number: $port\n";
         return 1;
     }
 
-    my $paddr = sockaddr_in($port, $iaddr);
-    my $proto   = getprotobyname('tcp') || 6;
+    # We fire up a new sockfilt to do the data tranfer for us.
+    # FIX: make it use IPv6 if need be
+    $slavepid = open2(\*DREAD, \*DWRITE,
+                      "./server/sockfilt --connect $port --logfile log/sockdata$ftpdnum$ext.log --pidfile .sockdata$ftpdnum$ext.pid $ipv6");
 
-    socket(SOCK, PF_INET, SOCK_STREAM, $proto) || die "major failure";
-    connect(SOCK, $paddr)    || return 1;
+    print DWRITE "PING\n";
+    my $pong;
+    sysread DREAD, $pong, 5;
 
-    return \&SOCK;
+    if($pong !~ /^PONG/) {
+        logmsg "Failed sockfilt for data connection\n";
+        kill(9, $slavepid);
+    }
+
+    logmsg "====> Client DATA connect to port $port\n";
+
+    return;
 }
 
-$SIG{CHLD} = \&REAPER;
-
 my %customreply;
 my %customcount;
 my %delayreply;
 sub customize {
     undef %customreply;
+
+    $nosave = 0; # default is to save as normal
+    $controldelay = 0; # default is no delaying the responses
+
     open(CUSTOM, "<log/ftpserver.cmd") ||
         return 1;
 
@@ -505,23 +721,40 @@ sub customize {
     while(<CUSTOM>) {
         if($_ =~ /REPLY ([A-Z]+) (.*)/) {
             $customreply{$1}=$2;
+            logmsg "FTPD: set custom reply for $1\n";
         }
         if($_ =~ /COUNT ([A-Z]+) (.*)/) {
             # we blank the customreply for this command when having
             # been used this number of times
             $customcount{$1}=$2;
+            logmsg "FTPD: blank custom reply for $1 after $2 uses\n";
         }
         elsif($_ =~ /DELAY ([A-Z]+) (\d*)/) {
             $delayreply{$1}=$2;
+            logmsg "FTPD: delay reply for $1 with $2 seconds\n";
+        }
+        elsif($_ =~ /SLOWDOWN/) {
+            $controldelay=1;
+            logmsg "FTPD: send response with 0.1 sec delay between each byte\n";
         }
         elsif($_ =~ /RETRWEIRDO/) {
-            print "instructed to use RETRWEIRDO\n";
+            logmsg "FTPD: instructed to use RETRWEIRDO\n";
             $retrweirdo=1;
         }
         elsif($_ =~ /RETRNOSIZE/) {
-            print "instructed to use RETRNOSIZE\n";
+            logmsg "FTPD: instructed to use RETRNOSIZE\n";
             $retrnosize=1;
         }
+        elsif($_ =~ /PASVBADIP/) {
+            logmsg "FTPD: instructed to use PASVBADIP\n";
+            $pasvbadip=1;
+        }
+        elsif($_ =~ /NOSAVE/) {
+            # don't actually store the file we upload - to be used when
+            # uploading insanely huge amounts
+            $nosave = 1;
+            logmsg "FTPD: NOSAVE prevents saving of uploaded data\n";
+        }
     }
     close(CUSTOM);
 }
@@ -533,25 +766,32 @@ my @welcome=(
             '220-  | (__| |_| |  _ <| |___ '."\r\n",
             '220    \___|\___/|_| \_\_____|'."\r\n");
 
-for ( $waitedpid = 0;
-      ($paddr = accept(Client,Server)) || $waitedpid;
-        $waitedpid = 0, close Client)
-{
-    next if $waitedpid and not $paddr;
-    my($port,$iaddr) = sockaddr_in($paddr);
-    my $name = gethostbyaddr($iaddr,AF_INET);
+
+while(1) {
+    #
+    # We read 'sockfilt' commands.
+    # 
+    my $input;
+
+    logmsg "Awaiting input\n";
+    sysread_or_die(\*SFREAD, \$input, 5);
+
+    if($input !~ /^CNCT/) {
+        # we wait for a connected client
+        logmsg "sockfilt said: $input";
+        next;
+    }
+    logmsg "====> Client connect\n";
 
     # flush data:
     $| = 1;
+
+    kill(9, $slavepid) if($slavepid);
+    $slavepid=0;
         
-    logmsg "connection from $name [", inet_ntoa($iaddr), "] at port $port\n";
-    
-    open(STDIN,  "<&Client")   || die "can't dup client to stdin";
-    open(STDOUT, ">&Client")   || die "can't dup client to stdout";
-    
     &customize(); # read test control instructions
 
-    print @welcome;
+    sendcontrol @welcome;
     if($verbose) {
         for(@welcome) {
             print STDERR "OUT: $_";
@@ -560,8 +800,30 @@ for ( $waitedpid = 0;
     my $state="fresh";
 
     while(1) {
+        my $i;
 
-        last unless defined ($_ = <STDIN>);
+        # Now we expect to read DATA\n[hex size]\n[prot], where the [prot]
+        # part only is FTP lingo.
+
+        # COMMAND
+        sysread_or_die(\*SFREAD, \$i, 5);
+
+        if($i !~ /^DATA/) {
+            logmsg "sockfilt said $i";
+            if($i =~ /^DISC/) {
+                # disconnect
+                last;
+            }
+            next;
+        }
+
+        # SIZE of data
+        sysread_or_die(\*SFREAD, \$i, 5);
+
+        my $size = hex($i);
+
+        # data
+        sysread SFREAD, $_, $size;
         
         ftpmsg $_;
         
@@ -569,15 +831,14 @@ for ( $waitedpid = 0;
         s/[\n\r]+$//;
 
         unless (m/^([A-Z]{3,4})\s?(.*)/i) {
-            print "500 '$_': command not understood.\r\n";
-            logmsg "unknown crap received, bailing out hard\n";
+            sendcontrol "500 '$_': command not understood.\r\n";
             last;
         }
         my $FTPCMD=$1;
         my $FTPARG=$2;
         my $full=$_;
                  
-        logmsg "Received \"$full\"\n";
+        logmsg "< \"$full\"\n";
 
         if($verbose) {
             print STDERR "IN: $full\n";
@@ -585,7 +846,7 @@ for ( $waitedpid = 0;
 
         my $ok = $commandok{$FTPCMD};
         if($ok !~ /$state/) {
-            print "500 $FTPCMD not OK in state: $state!\r\n";
+            sendcontrol "500 $FTPCMD not OK in state: $state!\r\n";
             next;
         }
 
@@ -604,6 +865,7 @@ for ( $waitedpid = 0;
         my $delay = $delayreply{$FTPCMD};
         if($delay) {
             # just go sleep this many seconds!
+            logmsg("Sleep for $delay seconds\n");
             sleep($delay);
         }
 
@@ -618,10 +880,9 @@ for ( $waitedpid = 0;
                 # used enough number of times, now blank the customreply
                 $customreply{$FTPCMD}="";
             }
-            logmsg "$FTPCMD made to send '$text'\n";
         }
         if($text) {
-            print "$text\r\n";
+            sendcontrol "$text\r\n";
         }
 
         if($fake eq "") {
@@ -630,10 +891,14 @@ for ( $waitedpid = 0;
             my $func = $commandfunc{$FTPCMD};
             if($func) {
                 # it is!
-                \&$func($FTPARG, $FTPCMD);
+                &$func($FTPARG, $FTPCMD);
             }
         }
             
     } # while(1)
-    close(Client);
+    logmsg "====> Client disconnected\n";
 }
+
+print SFWRITE "QUIT\n";
+waitpid $sfpid, 0;
+exit;
