@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sws.c,v 1.89 2006-07-19 15:26:28 yangtse Exp $
+ * $Id: sws.c,v 1.102 2007-02-19 03:59:41 yangtse Exp $
  ***************************************************************************/
 
 /* sws.c: simple (silly?) web server
@@ -90,6 +90,7 @@ bool prevbounce;    /* instructs the server to increase the part number for
 
 struct httprequest {
   char reqbuf[REQBUFSIZ]; /* buffer area for the incoming request */
+  int checkindex; /* where to start checking of the request */
   int offset;     /* size of the incoming request */
   long testno;     /* test number found in the request */
   long partno;     /* part number found in the request */
@@ -100,7 +101,8 @@ struct httprequest {
   size_t cl;      /* Content-Length of the incoming request */
   bool digest;    /* Authorization digest header found */
   bool ntlm;      /* Authorization ntlm header found */
-
+  int pipe;       /* if non-zero, expect this many requests to do a "piped"
+                     request/response */
   int rcmd;       /* doing a special command, see defines above */
 };
 
@@ -188,14 +190,17 @@ static void sigpipe_handler(int sig)
 
 int ProcessRequest(struct httprequest *req)
 {
-  char *line=req->reqbuf;
+  char *line=&req->reqbuf[req->checkindex];
   char chunked=FALSE;
   static char request[REQUEST_KEYWORD_SIZE];
   static char doc[MAXDOCNAMELEN];
   char logbuf[256];
   int prot_major, prot_minor;
   char *end;
-  end = strstr(req->reqbuf, END_OF_HEADERS);
+  int error;
+  end = strstr(line, END_OF_HEADERS);
+
+  logmsg("ProcessRequest() called");
 
   /* try to figure out the request characteristics as soon as possible, but
      only once! */
@@ -229,7 +234,7 @@ int ProcessRequest(struct httprequest *req)
         return 1; /* done */
       }
 
-      if(!strncmp("/quit", ptr, 15)) {
+      if(!strncmp("/quit", ptr, 5)) {
         logmsg("Request-to-quit received");
         req->testno = DOCNUMBER_QUIT;
         return 1; /* done */
@@ -238,7 +243,7 @@ int ProcessRequest(struct httprequest *req)
       ptr++; /* skip the slash */
 
       /* skip all non-numericals following the slash */
-      while(*ptr && !isdigit((int)*ptr))
+      while(*ptr && !ISDIGIT(*ptr))
         ptr++;
 
       req->testno = strtol(ptr, &ptr, 10);
@@ -259,6 +264,9 @@ int ProcessRequest(struct httprequest *req)
 
       stream=fopen(filename, "rb");
       if(!stream) {
+        error = ERRNO;
+        logmsg("fopen() failed with error: %d %s", error, strerror(error));
+        logmsg("Error opening file: %s", filename);
         logmsg("Couldn't open test file %d", req->testno);
         req->open = FALSE; /* closes connection */
         return 1; /* done */
@@ -266,6 +274,7 @@ int ProcessRequest(struct httprequest *req)
       else {
         char *cmd = NULL;
         size_t cmdsize = 0;
+        int num=0;
 
         /* get the custom server control "commands" */
         cmd = (char *)spitout(stream, "reply", "servercmd", &cmdsize);
@@ -287,6 +296,11 @@ int ProcessRequest(struct httprequest *req)
             logmsg("instructed to stream");
             req->rcmd = RCMD_STREAM;
           }
+          else if(1 == sscanf(cmd, "pipe: %d", &num)) {
+            logmsg("instructed to allow a pipe size %d", num);
+            req->pipe = num-1; /* decrease by one since we don't count the
+                                  first request in this number */
+          }
           free(cmd);
         }
       }
@@ -323,9 +337,17 @@ int ProcessRequest(struct httprequest *req)
     }
   }
 
-  if(!end)
+  if(!end) {
     /* we don't have a complete request yet! */
+    logmsg("ProcessRequest returned without a complete request");
     return 0;
+  }
+  logmsg("ProcessRequest found a complete request");
+
+  if(req->pipe)
+    /* we do have a full set, advance the checkindex to after the end of the
+       headers, for the pipelining case mostly */
+    req->checkindex += (end - line) + strlen(END_OF_HEADERS);
 
   /* **** Persistancy ****
    *
@@ -402,6 +424,17 @@ int ProcessRequest(struct httprequest *req)
   if(strstr(req->reqbuf, "Connection: close"))
     req->open = FALSE; /* close connection after this request */
 
+  while(req->pipe) {
+    /* scan for more header ends within this chunk */
+    line = &req->reqbuf[req->checkindex];
+    end = strstr(line, END_OF_HEADERS);
+    if(!end)
+      break;
+    req->checkindex += (end - line) + strlen(END_OF_HEADERS);
+    req->pipe--;
+  }
+
+
   /* If authentication is required and no auth was provided, end now. This
      makes the server NOT wait for PUT/POST data and you can then make the
      test case send a rejection before any such data has been sent. Test case
@@ -415,25 +448,53 @@ int ProcessRequest(struct httprequest *req)
     else
       return 0; /* not complete yet */
   }
+
   return 1; /* done */
 }
 
 /* store the entire request in a file */
 void storerequest(char *reqbuf)
 {
+  int error;
+  ssize_t written;
+  ssize_t writeleft;
+  ssize_t totalsize;
   FILE *dump;
 
-  dump = fopen(REQUEST_DUMP, "ab"); /* b is for windows-preparing */
-  if(dump) {
-    size_t len = strlen(reqbuf);
-    fwrite(reqbuf, 1, len, dump);
+  if (reqbuf == NULL)
+    return;
 
-    fclose(dump);
-    logmsg("Wrote request (%d bytes) input to " REQUEST_DUMP,
-           (int)len);
+  totalsize = (ssize_t)strlen(reqbuf);
+  if (totalsize == 0)
+    return;
+
+  do {
+    dump = fopen(REQUEST_DUMP, "ab");
+  } while ((dump == NULL) && ((error = ERRNO) == EINTR));
+  if (dump == NULL) {
+    logmsg("Error opening file %s error: %d", REQUEST_DUMP, error);
+    logmsg("Failed to write request input to " REQUEST_DUMP);
+    return;
+  }
+
+  writeleft = totalsize;
+  do {
+    written = (ssize_t)fwrite((void *) &reqbuf[totalsize-writeleft],
+                              1, (size_t)writeleft, dump);
+    if (written > 0)
+      writeleft -= written;
+  } while ((writeleft > 0) && ((error = ERRNO) == EINTR));
+
+  fclose(dump);  /* close it ASAP */
+
+  if (writeleft > 0) {
+    logmsg("Error writing file %s error: %d", REQUEST_DUMP, error);
+    logmsg("Wrote only (%d bytes) of (%d bytes) request input to %s",
+           totalsize-writeleft, totalsize, REQUEST_DUMP);
   }
   else {
-    logmsg("Failed to write request input to " REQUEST_DUMP);
+    logmsg("Wrote request (%d bytes) input to " REQUEST_DUMP,
+           totalsize);
   }
 }
 
@@ -450,6 +511,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
   req->testno = DOCNUMBER_NOTHING; /* safe default */
   req->open = TRUE; /* connection should remain open and wait for more
                        commands */
+  req->pipe = 0;
 
   /*** end of httprequest init ***/
 
@@ -457,7 +519,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
     ssize_t got = sread(sock, reqbuf + req->offset, REQBUFSIZ - req->offset);
     if (got <= 0) {
       if (got < 0) {
-        logmsg("recv() returned error: %d", errno);
+        logmsg("recv() returned error: %d", SOCKERRNO);
         return DOCNUMBER_INTERNAL;
       }
       logmsg("Connection closed by client");
@@ -467,12 +529,20 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
       storerequest(reqbuf);
       return DOCNUMBER_INTERNAL;
     }
+
+    logmsg("Read %d bytes", got);
+
     req->offset += got;
 
     reqbuf[req->offset] = 0;
 
-    if(ProcessRequest(req))
+    if(ProcessRequest(req)) {
+      if(req->pipe--) {
+        logmsg("Waiting for another piped request");
+        continue;
+      }
       break;
+    }
   }
 
   if (req->offset >= REQBUFSIZ) {
@@ -503,6 +573,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   FILE *dump;
   int persistant = TRUE;
   size_t responsesize;
+  int error;
 
   static char weare[256];
 
@@ -533,6 +604,9 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   req->open = FALSE;
 
   if(req->testno < 0) {
+    size_t msglen;
+    char msgbuf[64];
+
     switch(req->testno) {
     case DOCNUMBER_QUIT:
       logmsg("Replying to QUIT");
@@ -541,8 +615,10 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
     case DOCNUMBER_WERULEZ:
       /* we got a "friends?" question, reply back that we sure are */
       logmsg("Identifying ourselves as friends");
-      sprintf(weare, "HTTP/1.1 200 OK\r\n\r\nWE ROOLZ: %d\r\n",
-              (int)getpid());
+      sprintf(msgbuf, "WE ROOLZ: %d\r\n", (int)getpid());
+      msglen = strlen(msgbuf);
+      sprintf(weare, "HTTP/1.1 200 OK\r\nContent-Length: %d\r\n\r\n%s",
+              msglen, msgbuf);
       buffer = weare;
       break;
     case DOCNUMBER_INTERNAL:
@@ -575,6 +651,9 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
 
     stream=fopen(filename, "rb");
     if(!stream) {
+      error = ERRNO;
+      logmsg("fopen() failed with error: %d %s", error, strerror(error));
+      logmsg("Error opening file: %s", filename);
       logmsg("Couldn't open test file");
       return 0;
     }
@@ -587,6 +666,9 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
     /* re-open the same file again */
     stream=fopen(filename, "rb");
     if(!stream) {
+      error = ERRNO;
+      logmsg("fopen() failed with error: %d %s", error, strerror(error));
+      logmsg("Error opening file: %s", filename);
       logmsg("Couldn't open test file");
       return 0;
     }
@@ -599,6 +681,9 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
 
   dump = fopen(RESPONSE_DUMP, "ab"); /* b is for windows-preparing */
   if(!dump) {
+    error = ERRNO;
+    logmsg("fopen() failed with error: %d %s", error, strerror(error));
+    logmsg("Error opening file: %s", RESPONSE_DUMP);
     logmsg("couldn't create logfile: " RESPONSE_DUMP);
     return -1;
   }
@@ -686,6 +771,7 @@ int main(int argc, char *argv[])
   char *pidname= (char *)".http.pid";
   struct httprequest req;
   int rc;
+  int error;
   int arg=1;
 #ifdef CURL_SWS_FORK_ENABLED
   bool use_fork = FALSE;
@@ -734,7 +820,7 @@ int main(int argc, char *argv[])
     }
   }
 
-#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
+#ifdef WIN32
   win32_init();
   atexit(win32_cleanup);
 #else
@@ -759,16 +845,16 @@ int main(int argc, char *argv[])
 #endif
 
   if (CURL_SOCKET_BAD == sock) {
-    logmsg("Error opening socket: %d", errno);
-    exit(1);
+    logmsg("Error opening socket: %d", SOCKERRNO);
+    return 1;
   }
 
   flag = 1;
-  if (0 != setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
+  if (0 != setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
             (void *) &flag, sizeof(flag))) {
-    logmsg("setsockopt(SO_REUSEADDR) failed: %d", errno);
+    logmsg("setsockopt(SO_REUSEADDR) failed: %d", SOCKERRNO);
     sclose(sock);
-    exit(1);
+    return 1;
   }
 
 #ifdef ENABLE_IPV6
@@ -790,9 +876,9 @@ int main(int argc, char *argv[])
   }
 #endif /* ENABLE_IPV6 */
   if(0 != rc) {
-    logmsg("Error binding socket: %d", errno);
+    logmsg("Error binding socket: %d", SOCKERRNO);
     sclose(sock);
-    exit(1);
+    return 1;
   }
 
   pidfile = fopen(pidname, "w");
@@ -801,9 +887,12 @@ int main(int argc, char *argv[])
     fclose(pidfile);
   }
   else {
-    fprintf(stderr, "Couldn't write pid file\n");
+    error = ERRNO;
+    logmsg("fopen() failed with error: %d %s", error, strerror(error));
+    logmsg("Error opening file: %s", pidname);
+    logmsg("Couldn't write pid file");
     sclose(sock);
-    exit(1);
+    return 1;
   }
 
   logmsg("Running IPv%d version on port %d",
@@ -817,16 +906,16 @@ int main(int argc, char *argv[])
   /* start accepting connections */
   rc = listen(sock, 5);
   if(0 != rc) {
-    logmsg("listen() failed with error: %d", errno);
+    logmsg("listen() failed with error: %d", SOCKERRNO);
     sclose(sock);
-    exit(1);
+    return 1;
   }
 
   while (1) {
     msgsock = accept(sock, NULL, NULL);
 
     if (CURL_SOCKET_BAD == msgsock) {
-      printf("MAJOR ERROR: accept() failed with error: &d\n", errno);
+      printf("MAJOR ERROR: accept() failed with error: %d\n", SOCKERRNO);
       break;
     }
 
