@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: util.c,v 1.7 2006-05-24 21:39:52 bagder Exp $
+ * $Id: util.c,v 1.16 2007-02-19 02:04:02 yangtse Exp $
  ***************************************************************************/
 #include "setup.h" /* portability help from the lib directory */
 
@@ -27,8 +27,6 @@
 #include <string.h>
 #include <stdarg.h>
 #include <signal.h>
-#include <time.h>
-#include <sys/time.h>
 #include <sys/types.h>
 
 #ifdef HAVE_UNISTD_H
@@ -54,19 +52,11 @@
 #include "curlx.h" /* from the private lib dir */
 #include "getpart.h"
 #include "util.h"
+#include "timeval.h"
 
-/*
- * ourerrno() returns the errno (or equivalent) on this platform to
- * hide platform specific for the function that calls this.
- */
-int ourerrno(void)
-{
-#ifdef WIN32
-  return (int)GetLastError();
-#else
-  return errno;
+#if defined(ENABLE_IPV6) && defined(__MINGW32__)
+const struct in6_addr in6addr_any = {{ IN6ADDR_ANY_INIT }};
 #endif
-}
 
 /* someone else must set this properly */
 extern const char *serverlogfile;
@@ -74,15 +64,23 @@ extern const char *serverlogfile;
 void logmsg(const char *msg, ...)
 {
   va_list ap;
-  char buffer[256]; /* possible overflow if you pass in a huge string */
+  char buffer[512]; /* possible overflow if you pass in a huge string */
   FILE *logfp;
-
-  struct timeval tv = curlx_tvnow();
-  time_t sec = tv.tv_sec;
-  struct tm *now =
-    localtime(&sec); /* not multithread safe but we don't care */
-
+  int error;
+  struct timeval tv;
+  time_t sec;
+  struct tm *now;
   char timebuf[20];
+
+  if (!serverlogfile) {
+    fprintf(stderr, "Error: serverlogfile not set\n");
+    return;
+  }
+
+  tv = curlx_tvnow();
+  sec = tv.tv_sec;
+  now = localtime(&sec); /* not multithread safe but we don't care */
+
   snprintf(timebuf, sizeof(timebuf), "%02d:%02d:%02d.%06ld",
            now->tm_hour, now->tm_min, now->tm_sec, tv.tv_usec);
 
@@ -95,14 +93,21 @@ void logmsg(const char *msg, ...)
     fprintf(logfp, "%s %s\n", timebuf, buffer);
     fclose(logfp);
   }
+  else {
+    error = ERRNO;
+    fprintf(stderr, "fopen() failed with error: %d %s\n",
+            error, strerror(error));
+    fprintf(stderr, "Error opening file: %s\n", serverlogfile);
+    fprintf(stderr, "Msg not logged: %s %s\n", timebuf, buffer);
+  }
 }
 
-#if defined(WIN32) && !defined(__CYGWIN__)
+#ifdef WIN32
 /* use instead of perror() on generic windows */
 void win32_perror (const char *msg)
 {
-  char buf[256];
-  DWORD err = WSAGetLastError();
+  char buf[512];
+  DWORD err = SOCKERRNO;
 
   if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, err,
                      LANG_NEUTRAL, buf, sizeof(buf), NULL))
@@ -111,30 +116,30 @@ void win32_perror (const char *msg)
      fprintf(stderr, "%s: ", msg);
   fprintf(stderr, "%s\n", buf);
 }
-#endif
+#endif  /* WIN32 */
 
-#if defined(WIN32) && !defined(__CYGWIN__)
+#ifdef USE_WINSOCK
 void win32_init(void)
 {
   WORD wVersionRequested;
   WSADATA wsaData;
   int err;
-  wVersionRequested = MAKEWORD(2, 0);
+  wVersionRequested = MAKEWORD(USE_WINSOCK, USE_WINSOCK);
 
   err = WSAStartup(wVersionRequested, &wsaData);
 
   if (err != 0) {
     perror("Winsock init failed");
-    logmsg("Error initialising winsock -- aborting\n");
+    logmsg("Error initialising winsock -- aborting");
     exit(1);
   }
 
-  if ( LOBYTE( wsaData.wVersion ) != 2 ||
-       HIBYTE( wsaData.wVersion ) != 0 ) {
+  if ( LOBYTE( wsaData.wVersion ) != USE_WINSOCK ||
+       HIBYTE( wsaData.wVersion ) != USE_WINSOCK ) {
 
     WSACleanup();
     perror("Winsock init failed");
-    logmsg("No suitable winsock.dll found -- aborting\n");
+    logmsg("No suitable winsock.dll found -- aborting");
     exit(1);
   }
 }
@@ -143,7 +148,7 @@ void win32_cleanup(void)
 {
   WSACleanup();
 }
-#endif
+#endif  /* USE_WINSOCK */
 
 /* set by the main code to point to where the test dir is */
 const char *path=".";
@@ -154,4 +159,3 @@ char *test2file(long testno)
   snprintf(filename, sizeof(filename), TEST_DATA_PATH, path, testno);
   return filename;
 }
-
